var socket;

var mainCanvas;
var mainCanvasContext;
var mainCanvasStage;

var backgroundCanvas;
var backgroundCanvasContext;

var menuView;

var showingMenu = false, settingsButtons = [], settings = {};

var tombstone;
var maxTombstones = 3;
var tombstones = [];
var crater;
var maxCraters = 3;
var craters = [];

var images = {};
var backgroundImage;

var soldier;
var ss;
var x = 100, y = 100;

var scaleFactor = 0.7;

var clientIDs = {};
var clients = [];
var localBullets = [];
var localCanonBalls = [];

var runClock = false;
var currentClock = 0;
var maxClock = 1;
var gameClock;

var musketSound = new Audio("Sound/musket.ogg");
var canonFireSound = new Audio("Sound/canon_fire.ogg");
var explosionSound = new Audio("Sound/explosion.ogg");
var dieSound = new Audio("Sound/die.ogg");
var swordSound = new Audio("Sound/sword.ogg");

var battleMusic = new Audio("Sound/battle.mp3");
var norwegianVictoryMusic = new Audio("Sound/no.mp3");
var swedishVictoryMusic = new Audio("Sound/se.mp3");

$(document).ready(function(){

  backgroundCanvas = document.getElementById('backgroundCanvas');
  backgroundCanvasContext = backgroundCanvas.getContext("2d");
  var backgroundImage = new Image();
  backgroundImage.onload = function() {
    backgroundCanvasContext.drawImage(backgroundImage, 0, 0);
  };
  backgroundImage.src = 'Images/background.png';

  tombstone = new Image();
  tombstone.src = 'Images/tombstone.png';

  crater = new Image();
  crater.src = 'Images/crater.png';

  mainCanvas = document.getElementById('mainCanvas');
  mainCanvasContext = mainCanvas.getContext("2d");
  
  //resize();

  mainCanvasStage = new createjs.Stage(document.getElementById("mainCanvas"));
  
  loadImages();


  playMusic(battleMusic);

  //mainCanvas.addEventListener("click", mouseClick, false);
});

function playMusic(music){
  if(!soundEnabled){
    return;
  }
  stopAllMusic();
  music.play();

  music.addEventListener('ended', function () {
    // Wait 500 milliseconds before next loop
    setTimeout(function () { music.play(); }, 500);
  }, false);
}

function stopAllMusic(){
  stopMusic(battleMusic);
  stopMusic(norwegianVictoryMusic);
  stopMusic(swedishVictoryMusic);
}

function stopMusic(music){
  if(music.duration > 0 && !music.paused){
    music.pause();
    music.currentTime = 0;
  }
}

function playSound(sound){
  if(!soundEnabled){
    return;
  }
  sound.currentTime=0;
  sound.play();
}

function createNewImageToImages(src, name, rect){
  var img = new Object();
  img.image = new Image();
  img.image.src = src;
  img.name = name;
  img.rect = rect;
  img.image.onload = function(){
    images[name] = img;
    ready();
  }
}

function loadImages(){
  images.readyImages = 0;
  images.numImages = 12;

  createNewImageToImages('Images/norsk_soldat ny.png', 'norwaysoldier', {x:0, y:2*64, width:40, height:64});
  createNewImageToImages('Images/norsk_kanon ny.png', 'norwaycanon', {x:0, y:0, width:96, height:64});
  createNewImageToImages('Images/norsk_rytter ny.png', 'norwayhorse', {x:0, y:2*84, width:110, height:84});
  createNewImageToImages('Images/svensk_soldat.png', 'swedensoldier', {x:0, y:2*64, width:40, height:64});
  createNewImageToImages('Images/svensk_kanon.png', 'swedencanon', {x:0, y:0, width:96, height:64});
  createNewImageToImages('Images/svensk_rytter.png', 'swedenhorse', {x:0, y:2*84, width:110, height:84});
  createNewImageToImages('Images/bullet.png', 'bullet', {x:0, y:0, width:0, height:0});
  createNewImageToImages('Images/canonball.png', 'canonBall', {x:0, y:0, width:0, height:0});
  createNewImageToImages('Images/arrowRight.png', 'arrowRight', {x:0, y:0, width:0, height:0});
  createNewImageToImages('Images/arrowLeft.png', 'arrowLeft', {x:0, y:0, width:0, height:0});
  createNewImageToImages('Images/sound.png', 'sound', {x:0, y:0, width:0, height:0});
  createNewImageToImages('Images/nosound.png', 'nosound', {x:0, y:0, width:0, height:0});

  // // create a BitmapAnimation instance to display and play back the sprite sheet:
   // images.norwegianSolderAnimation = new createjs.BitmapAnimation(images.norwegianSolderSpriteSheet);
  // // start playing the first sequence:
   // images.norwegianSolderAnimation.gotoAndPlay("walk");     //animate
}

function setupSocket(){
  //console.log('Connecting to server');
  connectToLocalServer();

  addBinding('verifyAcc', function(data){
    console.log('verifyAcc');
    socket.emit('gamewall');
  });

  addBinding('chooseGameMode', function(data){
    console.log('choose gamemode');
    showMenu();
    //socket.emit('chooseGameMode', prompt('GameMode: '));
  });

  // console.log('chooseGameMode');

  addBinding('setup', function(data){
    // console.log('setup');
    updateScaleFactor(data.scaleFactor);
    updateScore(data.score);
    updateExplodedCanonballs(data.canonExplosions);
    updateGraveyard(data.tombstones);
    for(var i = 0; i < data.players.length; i++){
      addClient(data.players[i]);
    }
  });
  
  addBinding('gameOver', function(data){
    // console.log('GAME OVER');
    mainCanvasStage.removeAllChildren();
  });
  
  addBinding('updateScore', function(data){
    // console.log('update score');
    updateScore(data);
  });
  
  addBinding('newPlayer', function(player){
    console.log('Receiving newPlayer');
    addClient(player);
  });

  addBinding('updatePlayers', function(players){
    updatePlayers(players);
  });

  addBinding('updateProjectiles', function(projectiles){
    // console.log('Updating projectiles');
    // console.log(projectiles);
    updateProjectiles(projectiles);
  });

  addBinding('killspree', function(obj){
    // console.log('Updating projectiles');
    // console.log(projectiles);
    showKillspree(obj);
  });


  addBinding('killPlayer', function(playerid){
    // console.log('Updating projectiles');
    // console.log(projectiles);
    removeClient(playerid);
  });

  addBinding('showName', function(playerid){
    showName(playerid);
  });

  addBinding('showNameStop', function(playerid){
    showNameStop(playerid);
  });

  addBinding('attack', function(playerid){
    // console.log('Attack!');
    attackAnimation(playerid);
  });

  addBinding('canonStart', function(playerid){
    // console.log('canon start');
    canonStart(playerid);
  });

  addBinding('canonEnd', function(playerid){
    // console.log('canon end');
    canonEnd(playerid);
  });


  addBinding('explodeCanonBall', function(pos){
    explodeAt(pos);
  });


  addBinding('showSummary', function(data){
    showSummary(data);
  });


  // socket.on('updateCanonBalls', function(canonBalls){
  //   updateCanonBalls(canonBalls);
  // });
}

var summaryView;
function createSummary(){
  summaryView = new createjs.Container();
  summaryView.x = 0;
  summaryView.y = 0;
  summaryView.width = mainCanvas.width;
  summaryView.height = mainCanvas.height;

  var buttonBackground = new createjs.Shape();
  buttonBackground.x = 0;
  buttonBackground.y = 0;
  buttonBackground.barwidth = summaryView.width;
  buttonBackground.barheight = summaryView.height;
  buttonBackground.graphics.clear().beginFill("#000000").drawRect(0, 0, buttonBackground.barwidth, buttonBackground.barheight);

  var summaryScoreText = new createjs.Text("Home", "30px Arial", "#FFF");
  summaryScoreText.textAlign = 'center';
  summaryScoreText.x = summaryView.width / 2;
  summaryScoreText.y = 50;
  summaryScoreText.name = 'scoreText';

  var norwayImg = new createjs.Bitmap('Images/norge.png');
  norwayImg.x = summaryScoreText.x - 170;
  norwayImg.y = summaryScoreText.y - 10;
  norwayImg.scaleX = 1;
  norwayImg.scaleY = 1;
  norwayImg.name = 'norwayImage';
  norwayImg.image.onload = function(){
    console.log('w: '+norwayImg.image.width);
    norwayImg.scaleX = 70 / norwayImg.image.width;
    norwayImg.scaleY = 50 / norwayImg.image.height;
    norwayImg.x -= norwayImg.image.width * norwayImg.scaleX;
  }

  var swedenImg = new createjs.Bitmap('Images/sverige.png');
  swedenImg.x = summaryScoreText.x + 170;
  swedenImg.y = summaryScoreText.y - 10;
  swedenImg.scaleX = 1;
  swedenImg.scaleY = 1;
  swedenImg.name = 'swedenImage';
  swedenImg.image.onload = function(){
    swedenImg.scaleX = 70 / swedenImg.image.width;
    swedenImg.scaleY = 50 / swedenImg.image.height;
  }

  



  width = 200;
  height = 100;
  x = summaryView.width - width;
  y = summaryView.height - height;
  var restartContainer = createSummaryButton('Restart', x, y, width, height, function(){
    startGame();
  });

  x = x - width - 10;
  var newGameContainer = createSummaryButton('New game', x, y, width, height, function(){
    showMenu();
    mainCanvasStage.removeChild(summaryView);
  });

  summaryView.addChild(buttonBackground, summaryScoreText, norwayImg, swedenImg, restartContainer, newGameContainer);


  var x = 50;
  var y = 200;
  var width = 400;
  var height = 120;
  createBestContainer('bestPlayer', x, y, width, height);
  /*
  createBestContainer('bestNorwegian', x, y, width, height);
  y = y + height;
  createBestContainer('bestSwedish', x, y, width, height);
  y = y + height;
  createBestContainer('bestSoldier', x, y, width, height);
  y = y + height;
  createBestContainer('bestHorse', x, y, width, height);
  y = y + height;
  createBestContainer('bestCanon', x, y, width, height);
  */
}

function createBestContainer(name, x, y, width, height){
  var bestContainer = new createjs.Container();
  bestContainer.x = x;
  bestContainer.y = y;
  bestContainer.width = width;
  bestContainer.height = height;
  bestContainer.name = name;

  var txt = new createjs.Text("hei", "30px Arial", "#FFF");
  txt.textAlign = 'left';
  txt.x = 120;
  txt.name = 'text';
  var bestImg = new createjs.Bitmap();
  bestImg.x = 0;
  bestImg.scaleX = 1;
  bestImg.scaleY = 1;
  bestImg.name = 'image';

  bestContainer.addChild(bestImg, txt);
  summaryView.addChild(bestContainer);
}

function addImageToBestContainer(img, bestContainer){
  var bestImg = bestContainer.getChildByName('image');
  bestImg.image = img.image;
  bestImg.sourceRect = new createjs.Rectangle(img.rect.x, img.rect.y, img.rect.width, img.rect.height);
}

function createSummaryButton(text, x, y, width, height, onPressFunction){
  var container = new createjs.Container();
  container.width = width;
  container.height = height;
  container.x = x;
  container.y = y;
  var background = new createjs.Shape();
  background.graphics.clear().beginFill("#123456").drawRect(0, 0, container.width, container.height);
  background.onPress = onPressFunction;
  var txt = new createjs.Text(text, "30px Arial", "#FFF");
  txt.textAlign = 'center';
  txt.x = container.width / 2;
  txt.y = container.height / 3;
  container.addChild(background, txt);
  return container;
}

function showSummary(data){
  console.log('showSummary');
  mainCanvasStage.removeAllChildren();
  summaryView.getChildByName('scoreText').text = data.homename + ' ' + data.homescore + ' - ' + data.awayscore + ' ' + data.awayname;
  summaryView.getChildByName('bestPlayer').getChildByName('text').text = 'Best player: ' + data.bestPlayer.name;

  var characterImage = getCharacterImage(data.bestPlayer.team, data.bestPlayer.character);
  addImageToBestContainer(characterImage, summaryView.getChildByName('bestPlayer'));

  mainCanvasStage.addChild(summaryView);
}

function showSummary2(data){
  console.log('showSummary2');
  //mainCanvasStage.removeAllChildren();
  /*
  if(data.homescore > data.awayscore){
    playMusic(norwegianVictoryMusic);
  }else{
    playMusic(swedishVictoryMusic);
  }
  */
  mainCanvasStage.removeAllChildren();
  summaryView.getChildByName('scoreText').text = data.homename + ' ' + data.homescore + ' - ' + data.awayscore + ' ' + data.awayname;
  summaryView.getChildByName('bestNorwegian').getChildByName('text').text = 'Best norwegian: ' + data.topNorway[0].name;
  summaryView.getChildByName('bestSwedish').getChildByName('text').text = 'Best swedish: ' + data.topSweden[0].name;
  summaryView.getChildByName('bestSoldier').getChildByName('text').text = 'Best soldier: ' + data.bestSoldier.name;
  summaryView.getChildByName('bestHorse').getChildByName('text').text = 'Best horse: ' + data.bestHorse.name;
  summaryView.getChildByName('bestCanon').getChildByName('text').text = 'Best canon: ' + data.bestCanon.name;

  var characterImage = getCharacterImage(data.topNorway[0].team, data.topNorway[0].character);
  addImageToBestContainer(characterImage, summaryView.getChildByName('bestNorwegian'));

  characterImage = getCharacterImage(data.topSweden[0].team, data.topSweden[0].character);
  addImageToBestContainer(characterImage, summaryView.getChildByName('bestSwedish'));

  characterImage = getCharacterImage(data.bestSoldier.team, 'soldier');
  addImageToBestContainer(characterImage, summaryView.getChildByName('bestSoldier'));

  characterImage = getCharacterImage(data.bestHorse.team, 'horse');
  addImageToBestContainer(characterImage, summaryView.getChildByName('bestHorse'));

  characterImage = getCharacterImage(data.bestCanon.team, 'canon');
  addImageToBestContainer(characterImage, summaryView.getChildByName('bestCanon'));

  mainCanvasStage.addChild(summaryView);
}

function getCharacterImage(team, character){
  var img = team + character;
  var imag = images[img];
  return images[img];
}

function updateScaleFactor(fac){
  // console.log('update scale factor');
  scaleFactor = fac;
}

function updateGraveyard(graves){
  for(var i = 0; i < graves.length; i++){
    var x = graves[i].x;
    var y = graves[i].y;
    var tombstoneImg = new createjs.Bitmap(tombstone);
    tombstoneImg.x = x;
    tombstoneImg.y = y;
    tombstoneImg.scaleX = scaleFactor;
    tombstoneImg.scaleY = scaleFactor;
    mainCanvasStage.addChildAt(tombstoneImg, 0);
    tombstones.push(tombstoneImg);
    if(tombstones.length > maxTombstones){
      var excessTombstone = tombstones[0];
      fadeOut(0, 10, excessTombstone);
      tombstones.splice(0, 1);
    }
    // backgroundCanvasContext.drawImage(tombstone, x, y, tombstone.width * scaleFactor, tombstone.height * scaleFactor);
  }
}

function fadeOut(tick, maxTick, img){
  tick++;
  var alpha = 1 - (tick / maxTick);
  img.alpha = alpha;
  //console.log('Tick ' + tick + '/' + maxTick + ', alpha: ' + alpha);
  //console.log(img);
  if(tick >= maxTick){
    mainCanvasStage.removeChild(img);
  }else{
    setTimeout(function() {
        fadeOut(tick, maxTick, img);
    }, 100);
  }
}

function updateExplodedCanonballs(canonExplosions){
  for(var i = 0; i < canonExplosions.length; i++){
    var x = canonExplosions[i].x;
    var y = canonExplosions[i].y;
    // grave
    var explosionImg = new createjs.Bitmap(crater);
    explosionImg.x = x;
    explosionImg.y = y;
    explosionImg.scaleX = scaleFactor;
    explosionImg.scaleY = scaleFactor;
    mainCanvasStage.addChildAt(explosionImg, 0);
    craters.push(explosionImg);
    if(craters.length > maxCraters){
      var excessCrater = craters[0];
      fadeOut(0, 10, excessCrater);
      craters.splice(0, 1);
    }
    // backgroundCanvasContext.drawImage(crater, x, y, crater.width * scaleFactor, crater.height * scaleFactor);
  }
}

function updateScore(data){
  gameGUI.getChildByName('score').text = data.homeName + ' ' + data.homeScore + ' - ' + data.awayScore + ' ' + data.awayName;
}

function getImage(character, team){
  if(team == 'norway'){ 
    if(character == 'canon'){
      return 'norwegianCanonSpriteSheet';
    }else if(character == 'horse'){
      return 'norwegianCavalerySpriteSheet';
    }else{
      return 'norwegianSolderSpriteSheet';
    }
  }else{
    if(character == 'canon'){
      return 'swedishCanonSpriteSheet';
    }else if(character == 'horse'){
      return 'swedishCavalerySpriteSheet';
    }else{
      return 'swedishSolderSpriteSheet';
    }
  }
}

function addClient(client){
  console.log('Adding client ' + client.character);
  if(clientIDs[client.id]){
    console.log('player already exists');
  }
  else{
    console.log('player does not exists');
  }
  client.img = new createjs.BitmapAnimation(images[getImage(client.character, client.team)]);
  //client.img = new createjs.BitmapAnimation(images['explosionSpriteSheet']);
  client.img.gotoAndPlay("die");
  client.img.x = client.x;
  client.img.y = client.y;
  client.img.scaleX = scaleFactor;
  client.img.scaleY = scaleFactor;
  mainCanvasStage.addChild(client.img);
  clientIDs[client.id] = client;
  clients.push(client);
}

function getAnimationOffset(client){
  var flip = 1;
  if(client.team == 'sweden'){
    flip = -1;
  }

  var offset = { x:0, y:0 };
  if(client.img.currentAnimation == 'fire'){
    offset.x = 6 * flip;
  }

  return offset;
}

function removeClient(clientid){

  var client = clientIDs[clientid];
  //clients.splice(clients.indexOf(client), 1);
  // console.log('killing ' + clientid);
  if(client){
    // console.log('Playing die animation for image ' + getImage(client.character, client.team));
    playSound(dieSound);
    mainCanvasStage.removeChild(client.img);

    if(client.character == 'canon'){
      canonEnd(client.id);
    }

    var ss = images[getImage(client.character, client.team)];
    var grave = new createjs.BitmapAnimation(ss);

    grave.gotoAndPlay('die');
    grave.onAnimationEnd = function(){
      var animData = ss.getAnimation('die');
      grave.gotoAndStop(animData.frames[animData.frames.length-1]);
    }

    grave.x = client.x;
    grave.y = client.y;
    grave.scaleX = scaleFactor;
    grave.scaleY = scaleFactor;
    mainCanvasStage.addChildAt(grave, 0);
    tombstones.push(grave);
    if(tombstones.length > maxTombstones){
      var excessTombstone = tombstones[0];
      fadeOut(0, 10, excessTombstone);
      tombstones.splice(0, 1);
    }
  }
  
}



function ready() {
    // console.log('Image loaded');

    images.readyImages += 1;
    // console.log('Ready images ' + images.readyImages + '/' + images.numImages);
    if(images.readyImages >= images.numImages){
      // create spritesheet and assign the associated data.
      //console.log(images['norwaycanon']);
      images.norwegianSolderSpriteSheet = new createjs.SpriteSheet({
          // image to use
          images: [images.norwaysoldier.image], 
          // width, height & registration point of each sprite
          frames: {width: 40, height: 64, regX: 20, regY: 32}, 
          animations: {    
              walk: [0, 7, "walk", 10],
              attack: [8, 15, "attack", 5],
              idle: [16, 17, "idle", 50],
              die: [24, 31, "die", 10],
              water: [32, 33, "water", 30]
          }
      });
      images['norwegianCanonSpriteSheet'] = new createjs.SpriteSheet({
          // image to use
          images: [images['norwaycanon'].image], 
          // width, height & registration point of each sprite
          frames: {width: 96, height: 64, regX: 48, regY: 32}, 
          animations: {    
              idle: [0, 0, "idle", 30],
              attack: [6, 11, "attack", 5],
              up: [12, 15, "up", 30],
              down: [18, 21, "down", 30],
              die: [24, 29, "die", 10]
          }
      });
      images.norwegianCavalerySpriteSheet = new createjs.SpriteSheet({
          // image to use
          images: [images.norwayhorse.image], 
          // width, height & registration point of each sprite
          frames: {width: 110, height: 84, regX: 55, regY: 42}, 
          animations: {    
              walk: [0, 5, "walk", 15],
              sprint: [0, 5, "sprint", 10],
              idle: [8, 8, "idle", 15],
              die: [16, 21, "die", 10],
              attack: [24, 29, "attack", 10],
              water: [32, 33, "water", 20]
          }
      });

      // create spritesheet and assign the associated data.
      images.swedishSolderSpriteSheet = new createjs.SpriteSheet({
          // image to use
          images: [images.swedensoldier.image], 
          // width, height & registration point of each sprite
          frames: {width: 40, height: 64, regX: 20, regY: 32}, 
          animations: {    
              walk: [0, 7, "walk", 10],
              attack: [8, 15, "attack", 5],
              idle: [16, 17, "idle", 50],
              die: [24, 31, "die", 10],
              water: [32, 33, "water", 30]
          }
      });
      images.swedishCanonSpriteSheet = new createjs.SpriteSheet({
          // image to use
          images: [images.swedencanon.image], 
          // width, height & registration point of each sprite
          frames: {width: 96, height: 64, regX: 48, regY: 32}, 
          animations: {    
              idle: [0, 0, "idle", 30],
              attack: [6, 11, "attack", 5],
              up: [12, 15, "up", 30],
              down: [18, 21, "down", 30],
              die: [24, 29, "die", 10]
          }
      });
      images.swedishCavalerySpriteSheet = new createjs.SpriteSheet({
          // image to use
          images: [images.swedenhorse.image], 
          // width, height & registration point of each sprite
          frames: {width: 110, height: 84, regX: 55, regY: 42}, 
          animations: {    
              walk: [0, 5, "walk", 15],
              sprint: [0, 5, "sprint", 10],
              idle: [8, 8, "idle", 15],
              die: [16, 21, "die", 10],
              attack: [24, 29, "attack", 10],
              water: [32, 33, "water", 20]
          }
      });
      images['explosionSpriteSheet'] = new createjs.SpriteSheet({
          // image to use
          images: ["Images/explosion.png"], 
          // width, height & registration point of each sprite
          frames: {width: 96, height: 96, regX: 48, regY: 48}, 
          animations: {    
              boom: [0, 5, "boom", 5]
          }
      });

      setupSocket();
      createGameGUI();
      createMenuView();
      createSummary();
 
      // addClient({ x: 0, y:0});

      // addClient({ x: 0, y:20});

      createjs.Ticker.setFPS(60);
      createjs.Ticker.addListener(window);  
    }
  }

  function attackAnimation(clientID){
    var localPlayer = clientIDs[clientID];
    if(localPlayer){
      localPlayer.img.gotoAndPlay("attack");
      localPlayer.img.onAnimationEnd = function(){
        localPlayer.img.gotoAndPlay("idle");
        localPlayer.img.onAnimationEnd = function(){}
      }

      if(localPlayer.character == 'soldier'){
        playSound(musketSound);
      }else if(localPlayer.character == 'horse'){
        playSound(swordSound);
      }else if(localPlayer.character == 'canon'){
        playSound(canonFireSound);
      }
      //mainCanvasStage.addChild(localPlayer.img);
    }else{
      // console.log('ERROR: Could not locate local player with id ' + clientID);
    }
  }

  function updatePlayers(players){
    for(var i = 0; i < players.length; i++){
      var playerInArray = players[i];
      var localPlayer = clientIDs[playerInArray[0]];
      // console.log(localPlayer);
      if(localPlayer){
        // console.log('Updating pos to ' + playerInArray[1] + ' ' + playerInArray[2]);
        localPlayer.x = parseInt(playerInArray[1]);
        localPlayer.y = parseInt(playerInArray[2]);
        localPlayer.movingDirection = playerInArray[3];
        localPlayer.inWater = playerInArray[4];
      }
    }
  }

  function updateProjectiles(projectiles){
    for(var i = 0; i < localBullets.length; i++){
      mainCanvasStage.removeChild(localBullets[i].img);
    }
    for(var i = 0; i < localCanonBalls.length; i++){
      mainCanvasStage.removeChild(localCanonBalls[i].img);
    }

    localBullets = projectiles[0];

    for(var i = 0; i < localBullets.length; i++){
      // console.log('New bullet pos: ' + localBullets[i].x + ' ' + localBullets[i].y);
      localBullets[i].img = new createjs.Bitmap(images.bullet.image);
      localBullets[i].img.x = localBullets[i].x;
      localBullets[i].img.y = localBullets[i].y;
      localBullets[i].img.scaleX = scaleFactor;
      localBullets[i].img.scaleY = scaleFactor;
      mainCanvasStage.addChild(localBullets[i].img);
    }
    
    localCanonBalls = projectiles[1];
    
    for(var i = 0; i < localCanonBalls.length; i++){
      // console.log('New canon ball pos: ' + localCanonBalls[i].x + ' ' + localCanonBalls[i].y);
      localCanonBalls[i].img = new createjs.Bitmap(images.canonBall.image);
      localCanonBalls[i].img.x = localCanonBalls[i].x;
      localCanonBalls[i].img.y = localCanonBalls[i].y;
      localCanonBalls[i].img.scaleX = scaleFactor;
      localCanonBalls[i].img.scaleY = scaleFactor;
      mainCanvasStage.addChild(localCanonBalls[i].img);
    }
  }
  
  function tick() {
    //console.log('dTime ' + createjs.Ticker.getInterval());
    if(runClock){
      currentClock += createjs.Ticker.getInterval();
      var totalseconds = Math.ceil((maxClock - currentClock) / 1000);
      var minutes = Math.floor(totalseconds / 60);
      var seconds = totalseconds % 60;
      var mZero = minutes < 10 ? '0' : '';
      var sZero = seconds < 10 ? '0' : '';
      //gameClock.text = mZero + minutes + ':' + sZero + seconds;
      gameGUI.getChildByName('gameClock').text = mZero + minutes + ':' + sZero + seconds;
    }
    for(var i = 0; i < clients.length; i++){
      var client = clients[i];
      // console.log('Current animation: ' + client.img.currentAnimation);

      if(client.movingDirection){
        var moveX = client.movingDirection[0];
        var moveY = client.movingDirection[1];
        var animToPlay = 'idle';
        if(!client.inWater){
          if(client.character == 'soldier'){
            if(moveX != 0 || moveY != 0){
                animToPlay = 'walk';
            }else{
                animToPlay = 'idle';
            }
          }else if(client.character == 'horse'){
            if(Math.abs(parseInt(moveX)) == 1){
                animToPlay = 'walk';
            }else if(Math.abs(parseInt(moveX)) == 2){
                animToPlay = 'sprint';
            }else{
                animToPlay = 'idle';
            }
          }else if(client.character == 'canon'){
            if(parseInt(moveY) < 0){
                animToPlay = 'down';
            }else if(parseInt(moveY) > 0){
                animToPlay = 'up';
            }
          }
        }else{
          animToPlay = 'water';
        }
        if(client.img.currentAnimation != 'attack'){
          if(client.img.currentAnimation != animToPlay){
            client.img.gotoAndPlay(animToPlay);
          }
        }
      }
      

      var offset = getAnimationOffset(client);
      //console.log('Offset: ' + offset.x + ' ' + offset.y);
      client.img.x = client.x + offset.x;
      client.img.y = client.y + offset.y;
    }

    for(var id in nameArray){
      var localPlayer = clientIDs[id];
      var txt = nameArray[id];
      txt.x = localPlayer.x - 20;
      txt.y = localPlayer.y - 80;
    }

    for(var id in killspreetext){
      var txt = killspreetext[id];
      txt.tick++;
      var fontsize = parseInt(txt.font.substring(0,2)) + 1;
      txt.font = fontsize + "px Arial";
      if(txt.tick > 48){
        delete killspreetext[id];
        mainCanvasStage.removeChild(txt);
      }
    }

    for(var id in progressBarArray){
      var shape = progressBarArray[id];
      shape.barwidth += 0.25;
      shape.graphics.clear();
      shape.graphics.setStrokeStyle(1);
      shape.graphics.beginFill("#000000").drawRect(-15 * scaleFactor, -15 * scaleFactor, 34.5 * scaleFactor, shape.barheight * scaleFactor);
      shape.graphics.beginFill("#66FF66").drawRect(-15 * scaleFactor, -15 * scaleFactor, shape.barwidth * scaleFactor, shape.barheight * scaleFactor);
      //console.log(shape.barwidth);
    }

    //Interpolere
    // for(var i = 0; i < localBullets.length; i++){
    //   var bullet = localBullets[i];
    //   bullet.img.x = bullet.x;
    //   bullet.img.y = bullet.y;
    // }
    // for(var i = 0; i < localCanonBalls.length; i++){
    //   var canonBall = localCanonBalls[i];
    //   canonBall.img.x = canonBall.x;
    //   canonBall.img.y = canonBall.y;
    // }
    
    mainCanvasStage.update();
  }


function resize(){
  // backgroundCanvas.width = backgroundCanvas.parentNode.clientWidth;
  // backgroundCanvas.height = backgroundCanvas.parentNode.clientHeight;
  // backgroundCanvas.css('position', 'absolute');
  // backgroundCanvas.css('top', '0');
  // backgroundCanvas.css('left', '0');

  // mainCanvas.width = mainCanvas.parentNode.clientWidth;
  // mainCanvas.height = mainCanvas.parentNode.clientHeight;
  // mainCanvas.css('position', 'absolute');
  // mainCanvas.css('top', '0');
  // mainCanvas.css('left', '0');
}

function explodeAt(pos){

  if(!pos.inWater){
    var x = pos.x - 40 * scaleFactor;
    var y = pos.y - 40 * scaleFactor;

    var craterImg = new createjs.Bitmap(crater);
    craterImg.x = x;
    craterImg.y = y;
    craterImg.scaleX = scaleFactor;
    craterImg.scaleY = scaleFactor;
    mainCanvasStage.addChildAt(craterImg, 0);
    craters.push(craterImg);
    if(craters.length > maxCraters){
      var excessCrater = craters[0];
      fadeOut(0, 10, excessCrater);
      craters.splice(0, 1);
    }
    // backgroundCanvasContext.drawImage(crater, x, y, crater.width * scaleFactor, crater.height * scaleFactor);
  }

  playSound(explosionSound);
  var explosion = new createjs.BitmapAnimation(images.explosionSpriteSheet);
  explosion.gotoAndPlay("boom");
  explosion.x = pos.x;
  explosion.y = pos.y;
  explosion.onAnimationEnd = function(){
    mainCanvasStage.removeChild(explosion);
  }
  mainCanvasStage.addChild(explosion);

}


var nameArray = {};
function showName(clientid){
  //console.log('showName, id: '+clientid);
  var localPlayer = clientIDs[clientid];
  var txt = new createjs.Text(localPlayer.name, "30px Arial", "#FFF");
  txt.x = localPlayer.x - 20 * scaleFactor;
  txt.y = localPlayer.y - 80 * scaleFactor;
  txt.id = clientid;
  nameArray[clientid] = txt;
  //console.log('name: '+txt);
  //txt.rotation = 20;
  //txt.outline = true;
  mainCanvasStage.addChild(txt);
}

function showNameStop(clientid){
  var txt = nameArray[clientid];
  delete nameArray[clientid];
  mainCanvasStage.removeChild(txt);
}

var killspreetext = {};
function showKillspree(obj){
  var playerid = obj.playerid;
  var text = obj.text;
  var localPlayer = clientIDs[playerid];
  var txt = new createjs.Text(text, "10px Arial", "#F00");
  txt.x = localPlayer.x - 20 * scaleFactor;
  txt.y = localPlayer.y - 80 * scaleFactor;
  txt.id = playerid;
  txt.tick = 0;
  killspreetext[playerid] = txt;
  //txt.rotation = 20;
  //txt.outline = true;
  mainCanvasStage.addChild(txt);
}

var progressBarArray = {};
function canonStart(playerid){
  var localPlayer = clientIDs[playerid];
  var shape = new createjs.Shape();
  shape.x = localPlayer.x;
  shape.y = localPlayer.y - 10 * scaleFactor;
  shape.barwidth = 0;
  shape.barheight = 10;
  shape.id = playerid;
  shape.graphics.clear().beginFill("#66FF66").drawRect(-10 * scaleFactor, -10 * scaleFactor, shape.barwidth * scaleFactor, shape.barheight * scaleFactor);
  progressBarArray[playerid] = shape;
  mainCanvasStage.addChildAt(shape);
}

function canonEnd(playerid){
  //console.log('canonstart');
  var shape = progressBarArray[playerid];
  delete progressBarArray[playerid];
  mainCanvasStage.removeChild(shape);
}

var soundEnabled = true;
function toggleSound(){

  soundEnabled = !soundEnabled;
  console.log('Toggle ' + soundEnabled);

  if(soundEnabled){
    gameGUI.noSoundButton.alpha = 0.0;
    playMusic(battleMusic);
  }else{
    stopAllMusic();
    gameGUI.noSoundButton.alpha = 1.0;
  }
}

var gameGUI;
function createGameGUI(){
  gameGUI = new createjs.Container();

  gameGUI.soundButton = new createjs.Bitmap(images.sound);
  gameGUI.soundButton.x = 1920 - 70;
  gameGUI.soundButton.y = 10;
  gameGUI.soundButton.onPress = toggleSound;
  gameGUI.addChild(gameGUI.soundButton);

  gameGUI.noSoundButton = new createjs.Bitmap(images.nosound);
  gameGUI.noSoundButton.x = 1920 - 70;
  gameGUI.noSoundButton.y = 10;
  gameGUI.noSoundButton.alpha = 0.0;
  gameGUI.addChild(gameGUI.noSoundButton);

  gameGUI.scoreText = new createjs.Text('Norge 0 - 0 Sverige', "30px Arial", "#FFF");
  gameGUI.scoreText.textAlign = 'center';
  gameGUI.scoreText.x = 1920/2;
  gameGUI.scoreText.y = 60;
  gameGUI.scoreText.name = 'score';
  gameGUI.addChild(gameGUI.scoreText);

  var gameClock = new createjs.Text('', "30px Arial", "#FFF");
  gameClock.textAlign = 'center';
  gameClock.x = 1920/2;
  gameClock.y = 10;
  gameClock.name = 'gameClock';
  gameGUI.addChild(gameClock);
}

function showGameGUI(){
  mainCanvasStage.addChild(gameGUI);
}


function createMenuView(){
  menuView = new createjs.Container();

  settings.mode = {name: 'Game Mode', array: ['Team Deathmatch', 'Rounds'], index: 0};
  settings.level = {name: 'Level', array: ['Grasslands', 'Desert'], index: 0};
  settings.gameTime = {name: 'Time Limit', value: 2, max: 10, min: 1, delta: 1};
  settings.lives = {name: 'Lives', value: 1, max: 10, min: 1, delta: 1};
  settings.speed = {name: 'Game Speed', value: 100, max: 200, min: 10, delta: 10};
  settings.charSelection = {name: 'Character Selection', array: ['Random', 'Free', 'Force New'], index: 0};
  settings.ratioSelection = {name: 'Character Ratio', array: ['Free', 'Limited'], index: 0};

  createSetting(1920/2, 150, settings.mode);
  createSetting(1920/2, 250, settings.level);
  createSetting2(1920/2, 350, settings.gameTime);
  createSetting2(1920/2, 450, settings.lives);
  createSetting2(1920/2, 550, settings.speed);
  createSetting(1920/2, 650, settings.charSelection);
  createSetting(1920/2, 750, settings.ratioSelection);

  var buttonBackground = new createjs.Shape();
  buttonBackground.x = 1920/2;
  buttonBackground.y = 850;
  buttonBackground.barwidth = 500;
  buttonBackground.barheight = 70;
  buttonBackground.graphics.clear().beginFill("#000000").drawRect(-buttonBackground.barwidth/2, 0, buttonBackground.barwidth, buttonBackground.barheight);
  buttonBackground.onPress = startGame;
  menuView.addChild(buttonBackground);

  var buttonTxt = new createjs.Text('Start Game', "30px Arial", "#FFF");
  buttonTxt.textAlign = 'center';
  buttonTxt.x = 1920/2;
  buttonTxt.y = 850 + 35/2;
  menuView.addChild(buttonTxt);
}

function showMenu(){
  //console.log('Showing menu');
  showingMenu = true;
  mainCanvasStage.addChild(menuView);
  showGameGUI();
}

function createSetting(x, y, setting){
  var labelContainer = new createjs.Container();

  var txt = new createjs.Text(setting.name, "30px Arial", "#FFF");
  txt.textAlign = 'center';
  txt.x = x;
  txt.y = y;

  var varTxt = new createjs.Text(setting.array[setting.index], "30px Arial", "#FFF");
  varTxt.textAlign = 'center';
  varTxt.x = x;
  varTxt.y = y + 35;

  var background = new createjs.Shape();
  background.x = txt.x;
  background.y = txt.y;
  background.barwidth = 500;
  background.barheight = 70;
  background.graphics.clear().beginFill("#000000").drawRect(-background.barwidth/2, 0, background.barwidth, background.barheight);
  background.graphics.beginFill("#333333").drawRect(-background.barwidth/2, 0, background.barwidth, background.barheight/2);

  var leftArrow = new createjs.Bitmap(images.arrowLeft.image);
  leftArrow.x = x - background.barwidth/2;
  leftArrow.y = y;
  // leftArrow.onPress = menuLeftArrowClicked(varTxt, array, arrayIndex);
  leftArrow.onPress = function(){
    setting.index -= 1;
    if(setting.index < 0){
      setting.index = setting.array.length-1;
    }
    varTxt.text = setting.array[setting.index];
  };
  

  var rightArrow = new createjs.Bitmap(images.arrowRight.image);
  rightArrow.x = x + background.barwidth/2 - 45;
  rightArrow.y = y;
  // rightArrow.onPress = menuRightArrowClicked(varTxt, array, arrayIndex);
  rightArrow.onPress = function(){
    setting.index += 1;
    if(setting.index > setting.array.length-1){
      setting.index = 0;
    }
    varTxt.text = setting.array[setting.index];
  };
  
  labelContainer.addChild(background, leftArrow, rightArrow, txt, varTxt);
  menuView.addChild(labelContainer);
}

function createSetting2(x, y, setting){
  var labelContainer = new createjs.Container();

  var txt = new createjs.Text(setting.name, "30px Arial", "#FFF");
  txt.textAlign = 'center';
  txt.x = x;
  txt.y = y;

  var varTxt = new createjs.Text(setting.value, "30px Arial", "#FFF");
  varTxt.textAlign = 'center';
  varTxt.x = x;
  varTxt.y = y + 35;

  var background = new createjs.Shape();
  background.x = txt.x;
  background.y = txt.y;
  background.barwidth = 500;
  background.barheight = 70;
  background.graphics.clear().beginFill("#000000").drawRect(-background.barwidth/2, 0, background.barwidth, background.barheight);
  background.graphics.beginFill("#333333").drawRect(-background.barwidth/2, 0, background.barwidth, background.barheight/2);

  var leftArrow = new createjs.Bitmap(images.arrowLeft.image);
  leftArrow.x = x - background.barwidth/2;
  leftArrow.y = y;
  leftArrow.onPress = function(){
    setting.value -= setting.delta;
    if(setting.value < setting.min){
      setting.value = setting.min;
    }
    varTxt.text = setting.value;
  };
  

  var rightArrow = new createjs.Bitmap(images.arrowRight.image);
  rightArrow.x = x + background.barwidth/2 - 45;
  rightArrow.y = y;
  rightArrow.onPress = function(){
    setting.value += setting.delta;
    if(setting.value > setting.max){
      setting.value = setting.max;
    }
    varTxt.text = setting.value;
  };
  
  labelContainer.addChild(background, leftArrow, rightArrow, txt, varTxt);
  menuView.addChild(labelContainer);
}



function startGame(){
  var mode = settings.mode.array[settings.mode.index];
  var level = settings.level.array[settings.level.index];
  var time = settings.gameTime.value * 60; //send number of seconds
  var speed = settings.speed.value;
  var charSelection = settings.charSelection.array[settings.charSelection.index];
  var ratio = settings.ratioSelection.array[settings.ratioSelection.index];
  var lives = settings.lives.value - 1;
  var settingsMsg = { mode:mode, level: level, time:time, speed:speed, charSelection:charSelection, ratio:ratio, lives:lives };
  socket.emit('chooseGameMode', settingsMsg);

  showingMenu = false;
  mainCanvasStage.removeAllChildren();

  showGameGUI();
  //playMusic(battleMusic);

  maxClock = time*1000;
  gameGUI.getChildByName('gameClock').text = time;
  runClock = true;
  currentClock = 0;
}
