var Framework = require("../Framework/FIGAServer.js");

function App() {
    this.init = function() {
        App.prototype.init(__dirname);
    }
}

App.prototype = new Framework.GameWallServer;
var server = new App();






var bullets = [];
var canonBalls = [];
var speedFactor = 1.0;
var scaleFactor = 0.7;

var walkValue = 5;
var width = 1080;
var height = 1920;

var homeName = 'Norge';
var homeScore = 0;
var awayName = 'Sverige';
var awayScore = 0;

var canonExplosions = [];
var tombstones = [];

var waterStartX = 860;
var waterEndX = 980;
var bridgeStartY = 320;
var bridgeEndY = 390;
var bridge2StartY = 905;
var bridge2EndY = 975;

var gameMode = {
  isSelected: false,
  mode1: 'infinityLives',
  mode2: 'oneLife',
  chosenMode: 0
}
var gameSettings;

var gameMode2IntroTime = 3.0; //3 seconds
var gameMode2CurrentTime = 0.0; //Intro timer
var gameMode2Intro = true; //Are we in intro mode?

var gameTimer = 0;

/////////////////////////////////////////////////
//////         Add triggers here          ///////
/////////////////////////////////////////////////

//Gamewall connects override
server.addChannelBinding('gamewall', 'all',
  function() {
    if(!gameMode.isSelected){
      server.sendEventToSocket(this, 'chooseGameMode');
    }
    else{
      var players = [];
      server.getUserAppsConnected().forEach(function (socket) {
        if(socket.info){
          var player = {id:socket.info.id, name:socket.info.nickname, team: socket.info.team, movingDirection:socket.info.movingDirection, character:socket.info.character, x:socket.info.x, y:socket.info.y};
          players.push(player);
        }
      });

      var score = {homeName:homeName, homeScore:homeScore, awayName:awayName, awayScore:awayScore};
      server.sendDataToSocket(this, 'setup', {players:players, score:score, tombstones:tombstones, canonExplosions:canonExplosions, scaleFactor:scaleFactor});
    }
  }
);


server.addChannelBinding('verifyUserApp', 'all',
  function() {
    var hasUsername;
    if(this.info){
      hasUsername = true;
    }
    else{
      hasUsername = false;
    }
    server.sendDataToSocket(this, 'verifyAcc', {canConnect:gameMode.isSelected, hasUsername:hasUsername});
  }
);

server.addChannelBinding('verifyGameWallApp', 'all',
  function() {
    server.sendDataToSocket(this, 'verifyAcc', gameMode.isSelected);
  }
);



server.addChannelBinding('chooseGameMode', 'gamewall',
  function(settings) {

    gameMode.isSelected = true;
    gameSettings = settings;
    speedFactor = gameSettings.speed / 100;

    resetScore();
    resetPlayers();
    // console.log(settings);
    if(settings.mode == 'Team Deathmatch'){
      gameMode.chosenMode = 1;
      updateGameLogic = function(){
        // console.log('Setting mode 1');
        gameLogicMode1();
      }
    }
    else if(settings.mode == 'Rounds'){
      gameMode.chosenMode = 2;
      resetGameMode2();
      // console.log('Setting mode 2');
      updateGameLogic = function(){
        //gameLogicMode2();
        gameLogicMode2();
      }
    }

    var players = [];
    server.getUserAppsConnected().forEach(function (socket) {
      if(socket.info){
        var player = {id:socket.info.id, name:socket.info.nickname, team: socket.info.team, movingDirection:socket.info.movingDirection, character:socket.info.character, x:socket.info.x, y:socket.info.y};
        players.push(player);
      }
    });

    var score = {homeName:homeName, homeScore:homeScore, awayName:awayName, awayScore:awayScore};
    server.sendDataToSocket(this, 'setup', {players:players, score:score, tombstones:tombstones, canonExplosions:canonExplosions, scaleFactor:scaleFactor});
  }
);


function resetScore(){
  homeScore = 0;
  awayScore = 0;
    
}

function resetPlayers(){
  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      socket.info.numberOfKills = 0;
      socket.info.killData['soldier'] = 0;
      socket.info.killData['horse'] = 0;
      socket.info.killData['canon'] = 0;
      socket.info.numberOfTeamKills = 0;
      socket.info.numberOfDeaths = 0;
      socket.info.numberOfDeathsByOwnTeam = 0;
      socket.info.alive = true;
      socket.info.aliveTick = 0;
      var characterInfo = getCharacter(socket);
      var width = getWidth(characterInfo.character);
      var height = getHeight(characterInfo.character);
      socket.info.character = characterInfo.character;
      socket.info.playerWalkValueX = characterInfo.walkValueX;
      socket.info.playerWalkValueY = characterInfo.walkValueY;
      socket.info.originalPlayerWalkValueX = characterInfo.walkValueX;
      socket.info.originalPlayerWalkValueY = characterInfo.walkValueY;
      socket.info.movingDirection = [0, 0];
      socket.info.width = width;
      socket.info.height = height;
      var newStartPos = getStartPosition(socket.info.team, socket.info.character);
      socket.info.x = newStartPos[0];
      socket.info.y = newStartPos[1];
      server.sendDataToSocket(socket, 'setup', {team:socket.info.team, character:socket.info.character});
    }
  });  
}


var clientIDArray = [];

//Client connects override
server.addChannelBinding('client', 'all', function(nickname) {
    if(this.info){
      setupTriggersCharacter(this, this.info.character);
      server.sendDataToSocket(this, 'setup', {team:this.info.team, character:this.info.character});
    }
    else{
      //this.nickname = nickname;
      this.info = {id: this.id};
      this.info.nickname = nickname;
      if(gameSettings.charSelection == 'Free'){
        var team = getTeam(this);
        this.info.team = team;

        var activeSoldiers = 0, activeCavalery = 0, activeCanons = 0;
        var maxSoldiers = 'x', maxCavalery = 'x', maxCanons = 'x';
        if(gameSettings.ratio == 'Limited'){
          activeSoldiers = getUnitTypeForTeam(team, 'soldier');
          maxSoldiers = getMaxUnitType(team, 'soldier');
          activeCavalery = getUnitTypeForTeam(team, 'horse');
          maxCavalery = getMaxUnitType(team, 'horse');
          activeCanons = getUnitTypeForTeam(team, 'canon');
          maxCanons = getMaxUnitType(team, 'canon');
        }
        var msg = {team:team, activeSoldiers:activeSoldiers, maxSoldiers:maxSoldiers, activeCavalery:activeCavalery, maxCavalery:maxCavalery, activeCanons:activeCanons, maxCanons:maxCanons};
        server.sendDataToSocket(this, 'characterSelection', msg);
      }else{
        createNewPlayer(this);
        //console.log('nickname. '+this.info.nickname);
        server.sendDataToSocket(this, 'setup', {team:this.info.team, character:this.info.character});
        server.sendDataToChannel('newPlayer', {id:this.info.id, name:this.info.nickname, team:this.info.team, character:this.info.character, x:this.info.x, y:this.info.y, movingDirection:this.info.movingDirection}, 'gamewallApps');
      }
    }
});



server.addChannelBinding('characterSelected', 'userApp',
  function(selectedCharacter) {
    console.log('Creating selected character ' + selectedCharacter);
    createSelectedCharacter(this, selectedCharacter);
    server.sendDataToSocket(this, 'setup', {team:this.info.team, character:this.info.character});
    server.sendDataToChannel('newPlayer', {id:this.id, name:this.info.nickname, team:this.info.team, character:this.info.character, x:this.info.x, y:this.info.y, movingDirection:this.info.movingDirection}, 'gamewallApps');
  }
);


function getUnitTypeForTeam(team, character){
  var num = 0;

  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      if(socket.info.team == team && socket.info.character == character){
        num++;
      }
    }
  });

  return num;
}

function getMaxUnitType(team, character){
  var numberOfSameTeam = 0;
  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      if(socket.info.team == team){
        numberOfSameTeam++;
      }
    }
  });

  var percentOfType = 1.0; //100% percent atm
  var num = Math.floor(numberOfSameTeam * percentOfType);
  if(num <= 0){
    num = 1;
  }
  return num;
}

function createSelectedCharacter(socket, character){
  var team = socket.info.team;
  var characterInfo = getCharacterInfo(character);
  createPlayer(socket, characterInfo, team);
}

function createNewPlayer(socket){
  var team = getTeam(socket);
  var characterInfo = getCharacter(socket);
  createPlayer(socket, characterInfo, team);
}

function createPlayer(socket, characterInfo, team){
  var alive;
  var startPosition;
  var numLives = gameSettings.playerLives;

  if(gameMode.chosenMode == 1){
    alive = true;
    startPosition = getStartPosition(team,characterInfo.character);
  }else if(gameMode.chosenMode == 2){
    alive = false;
    numLives = 0;
    startPosition = getStartPosition(team,characterInfo.character);
    if(team == 'norway'){
      startPosition[0] = -50;
    }else{
      startPosition[0] = 1970;
    }
  }else{
    alive = true;
    startPosition = getStartPosition(team,characterInfo.character);
  }

  var teamVariable;
  if(team === 'norway'){
    teamVariable = 1;
  }
  else{
    teamVariable = -1;
  }
  var width = getWidth(characterInfo.character);
  var height = getHeight(characterInfo.character);

  socket.info = {
    id: socket.info.id,
    nickname: socket.info.nickname,
    team: team,
    teamVariable: teamVariable,
    movingDirection: [0, 0], //[dx, dy]
    character: characterInfo.character,
    x: startPosition[0],
    y: startPosition[1],
    alive: alive,
    aliveTick: 0,
    playerIsActive: false,
    playerWalkValueX: characterInfo.walkValueX,
    playerWalkValueY: characterInfo.walkValueY,
    attack: false,
    attackTick: 0,
    numberOfKills: 0,
    killData: {soldier:0, horse:0, canon:0},
    numberOfTeamKills: 0,
    numberOfDeaths: 0,
    numberOfDeathsByOwnTeam: 0,
    width: width,
    height: height,
    inWater: false,
    originalPlayerWalkValueX: characterInfo.walkValueX,
    originalPlayerWalkValueY: characterInfo.walkValueY,
    playerLives: numLives,
    upStart: function(){},
    leftStart: function(){},
    downStart: function(){},
    rightStart: function(){},
    directionStop: function(){},
    fireStart: function(){},
    fireStop: function(){},
    movePlayer: function(){}
  };

  setupTriggersCharacter(socket, characterInfo.character);
}

function setupTriggersCharacter(socket, character){
  if(character == 'soldier'){
    setupTriggersSoldier(socket);
  }else if(character == 'horse'){
    setupTriggersHorse(socket);
  }else if(character == 'canon'){
    setupTriggersCanon(socket);
  }else{
    console.log('ERROR: Trying to setup unknown character: ' + character);
    setupTriggersSoldier(socket);
  }
}

function getWidth(character){
  if(character === 'soldier'){
    return 32 * scaleFactor;
  }
  else if(character === 'horse'){
    return 110 * scaleFactor;
  }
  else{
    return 96 * scaleFactor;
  }
}

function getHeight(character){
  if(character === 'soldier'){
    return 64 * scaleFactor;
  }
  else if(character === 'horse'){
    return 84 * scaleFactor;
  }
  else{
    return 64 * scaleFactor;
  }
}

function getTeam(thisSocket){
  var numberOfNorwegian = 0;
  var numberOfSwedes = 0;
  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      if(thisSocket.id !== socket.info.id){
        if(socket.info.team === 'norway'){
          numberOfNorwegian++;
        }
        else{
          numberOfSwedes++;
        }
      }
    }
  });
  if(numberOfNorwegian > numberOfSwedes){
    return 'sweden';
  }
  else if(numberOfSwedes > numberOfNorwegian){
    return 'norway';
  }
  var random = Math.random();
  if(random < 0.5){
    return 'norway';
  }
  return 'sweden';
}

function getCharacter(socket){
  var random = Math.random();
  // removeTriggers(socket);
  if(random < 0.33333){
    // setupTriggersSoldier(socket);
    return getCharacterInfo('soldier');
  }
  else if(random < 0.66667){
    // setupTriggersHorse(socket);
    return getCharacterInfo('horse');
  }
  else{
    // setupTriggersCanon(socket);
    return getCharacterInfo('canon');
  }
}

function getCharacterInfo(character){
  if(character == 'soldier'){
    return {character:'soldier', walkValueX:4 * scaleFactor, walkValueY:4 * scaleFactor};
  }else if(character == 'horse'){
    return {character:'horse', walkValueX:6 * scaleFactor, walkValueY:6 * scaleFactor};
  }else if(character == 'canon'){
    return {character:'canon', walkValueX:0 * scaleFactor, walkValueY:2 * scaleFactor};
  }else{
    console.log('ERROR: Trying to get info for unknown character: ' + character);
    return {character:'soldier', walkValueX:4 * scaleFactor, walkValueY:4 * scaleFactor};
  }
}

function removeTriggers(socket){
  //console.log('removing triggers');
  if(socket.info){
    delete socket.info.upStart;
    delete socket.info.leftStart;
    delete socket.info.downStart;
    delete socket.info.rightStart;
    delete socket.info.directionStop;
    delete socket.info.fireStart;
    delete socket.info.fireStop;
    delete socket.info.movePlayer;
  }
  
}

function setupTriggersSoldier(socket){
  socket.info.upStart = function(){
    socket.info.movingDirection = [0,-1];
  }
  socket.info.downStart = function(){
    socket.info.movingDirection = [0,1];
  }
  socket.info.leftStart = function(){
    socket.info.movingDirection = [-1,0];
  }
  socket.info.rightStart = function(){
    socket.info.movingDirection = [1,0];
  }
  socket.info.directionStop = function(){
    socket.info.movingDirection = [0,0];
  }
  socket.info.fireStart = function (){
    fireSoldier(socket);
  }
  socket.info.fireStop = function (){
    
  }
  socket.info.movePlayer = function (){
    moveSoldier(socket);
  }
}

function setupTriggersHorse(socket){
  socket.info.upStart = function(){
    socket.info.movingDirection = [socket.info.teamVariable,-1];
  }
  socket.info.downStart = function(){
    socket.info.movingDirection = [socket.info.teamVariable,1];
  }
  socket.info.leftStart = function(){
    socket.info.movingDirection = [-1 + socket.info.teamVariable,0];
  }
  socket.info.rightStart = function(){
    socket.info.movingDirection = [1 + socket.info.teamVariable,0];
  }
  socket.info.directionStop = function(){
    socket.info.movingDirection = [socket.info.teamVariable,0];
  }
  socket.info.fireStart = function (){
    fireHorse(socket);
  }
  socket.info.fireStop = function (){
    
  }
  socket.info.movePlayer = function (){
    moveHorse(socket);
  }
}

function setupTriggersCanon(socket){
  socket.info.upStart = function(){
    socket.info.movingDirection = [0,-1];
  }
  socket.info.downStart = function(){
    socket.info.movingDirection = [0,1];
  }
  socket.info.leftStart = function(){
    socket.info.movingDirection = [0,0];
  }
  socket.info.rightStart = function(){
    socket.info.movingDirection = [0,0];
  }
  socket.info.directionStop = function(){
    socket.info.movingDirection = [0,0];
  }
  socket.info.fireStart = function (){
    fireStartCanon(socket);
  }
  socket.info.fireStop = function (){
    fireStopCanon(socket);
  }
  socket.info.movePlayer = function (){
    moveCanon(socket);
  }
}

function getStartPosition(team, character){
  var x = 300;
  var y = 200 + parseInt(Math.random() * (1080 - 400));
  if(character === 'soldier'){
    x += 300;
  }
  else if(character === 'horse'){
    x += 150;
  }

  if(team === 'sweden'){
    x = 1980 - x;
  }

  return [x, y];
}


server.addChannelBinding('fireStart', 'userApp',
  function() {
    if(!this.info.alive || this.info.attack || this.info.inWater){
      return;
    }
    this.info.attack = true;
    this.info.attackTick = 0;
    this.info.fireStart();
  }
);

server.addChannelBinding('fireStop', 'userApp',
  function() {
    if(!this.info.alive || this.info.inWater){
      return;
    }
    this.info.attack = true;
    //this.info.attackTick = 0;
    this.info.fireStop();
  }
);


server.addChannelBinding('activateHorse', 'userApp',
  function(data) {
    //console.log('activate horse!');
    if(!this.info.alive || this.info.attack){
      return;
    }
    this.info.playerIsActive = true;
  }
);


function fireSoldier(socket){
  //console.log('fire soldier!');
  server.sendDataToChannel('attack', socket.info.id, 'gamewallApps');
  socket.info.movingDirection = [0,0];
  var bullet = {
    id: socket.info.id,
    x: socket.info.x + socket.info.teamVariable * 20 * scaleFactor,
    y: socket.info.y - 15 * scaleFactor,
    speed: 15 * socket.info.teamVariable,
    tick: 0,
    timeToLive: 20 / speedFactor,
    width: 8 * scaleFactor,
    height: 8 * scaleFactor,
    team: socket.info.team
  };
  bullets.push(bullet);
}

function fireHorse(socket){
  //console.log('fire horse');
  server.sendDataToChannel('attack', socket.info.id, 'gamewallApps');  
  socket.info.movingDirection = [socket.info.teamVariable,0];
  var hitWidth = 32 * scaleFactor;
  var hitHeight = socket.info.height * scaleFactor;
  var hitCenter = {x: socket.info.x + ((socket.info.width / 2 + hitWidth) * socket.info.teamVariable), y: socket.info.y};
  server.getUserAppsConnected().forEach(function (opponent) {
    if(opponent.info){
      if(hitCenter.x - (hitWidth / 2) < opponent.info.x + (opponent.info.width / 2) && hitCenter.x + (hitWidth / 2)  > opponent.info.x - (opponent.info.width / 2)){
        if(hitCenter.y - (hitHeight / 2) < opponent.info.y + (opponent.info.height / 2) && hitCenter.y + (hitHeight / 2) > opponent.info.y - (opponent.info.height / 2)){
          if(socket.info.id !== opponent.id && opponent.info.alive && opponent.info.team !== socket.info.team){
            // console.log('we have a hit!');
            killPlayer(opponent);
            updateNumberOfKills(opponent, socket.info.id);
            updateOverallScore();
          }
        }
      }
    }
  }); 
}


var canonBallsTemp = [];
function fireStartCanon(socket){
  //console.log('fire start canon');
  server.sendDataToChannel('canonStart', socket.info.id, 'gamewallApps');
  socket.info.movingDirection = [0,0];
  var canonball = {
    id: socket.info.id,
    x: socket.info.x + socket.info.teamVariable * 20 * scaleFactor,
    y: socket.info.y - 5 * scaleFactor,
    speed: 25 * socket.info.teamVariable,
    tick: 0,
    timeToLive: 5,
    width: 96 * scaleFactor,
    height: 96 * scaleFactor,
    team: socket.info.team
  };
  canonBallsTemp.push(canonball);
}

function fireStopCanon(socket){
  // console.log('fire canon');
  for(var i = 0; i < canonBallsTemp.length; i++){
    var localCanonBall = canonBallsTemp[i];
    if(localCanonBall.id === socket.info.id){
      canonBallsTemp.splice(i,1);
      localCanonBall.timeToLive = localCanonBall.timeToLive / speedFactor;
      if(localCanonBall.timeToLive < 1){
        localCanonBall.timeToLive = 1;
      }
      canonBalls.push(localCanonBall);
      server.sendDataToChannel('attack', localCanonBall.id, 'gamewallApps');
      server.sendDataToChannel('canonEnd', localCanonBall.id, 'gamewallApps');
    }
  }
}

function updateCanonBallsTemp(){
  for(var i = 0; i < canonBallsTemp.length; i++){
    var localCanonBall = canonBallsTemp[i];
    localCanonBall.timeToLive += 1;
    if(localCanonBall.timeToLive > 60){
      canonBallsTemp.splice(i,1);
      localCanonBall.timeToLive = localCanonBall.timeToLive / speedFactor;
      if(localCanonBall.timeToLive < 1){
        localCanonBall.timeToLive = 1;
      }
      canonBalls.push(localCanonBall);
      server.sendDataToChannel('attack', localCanonBall.id, 'gamewallApps');
      server.sendDataToChannel('canonEnd', localCanonBall.id, 'gamewallApps');
    }
  }
}

server.addChannelBinding('controllerUpStart', 'userApp',
  function(word) {
    if(!this.info){
      return;
    }
    if(!this.info.alive || this.info.attack){
      return;
    }
    this.info.upStart();
  }
);

server.addChannelBinding('controllerLeftStart', 'userApp',
  function(word) {
    if(!this.info){
      return;
    }
    if(!this.info.alive || this.info.attack){
      return;
    }
    this.info.leftStart();
  }
);

server.addChannelBinding('controllerRightStart', 'userApp',
  function(word) {
    if(!this.info){
      return;
    }
    if(!this.info.alive || this.info.attack){
      return;
    }
    this.info.rightStart();
  }
);


server.addChannelBinding('controllerDownStart', 'userApp',
  function(word) {
    if(!this.info){
      return;
    }
    if(!this.info.alive || this.info.attack){
      return;
    }
    this.info.downStart();
  }
);


server.addChannelBinding('controllerDirectionStop', 'userApp',
  function(word) {
    //console.log('directionStop');
    if(!this.info){
      return;
    }
    if(!this.info.alive || this.info.attack){
      return;
    }
    this.info.directionStop();
  }
);




server.addChannelBinding('clientHelpStart', 'userApp',
  function(word) {
    if(!this.info){
      return;
    }
    if(!this.info.alive || this.info.attack){
      return;
    }
    //console.log('id: '+this.info.id);
    server.sendDataToChannel('showName', this.info.id, 'gamewallApps');
  }
);

server.addChannelBinding('clientHelpStop', 'userApp',
  function(word) {
    if(!this.info){
      return;
    }
    if(!this.info.alive || this.info.attack){
      return;
    }
    //console.log('id: '+this.info.id);
    server.sendDataToChannel('showNameStop', this.info.id, 'gamewallApps');
  }
);


/////////////////////////////////////////////////
//////             Game loop func         ///////
/////////////////////////////////////////////////

setInterval(function () {
  // console.log('game loop');
  //timeCounter += 24;
  if(gameMode.isSelected){
    gameTimer += (1/24);
    // console.log('Current time ' + gameTimer);
    if(gameTimer > gameSettings.time){
      console.log('Time out!');
      finishGame();
    }
  }
  
  updateGameLogic();
  //updateClients();
}, 1000/24);


function finishGame(){
  gameMode.isSelected = false;
  gameTimer = 0;
  //server.sendTriggerToSocket(this, 'chooseGameMode');
  updateGameLogic = function(){
    gameLogicMode0();
  }
  showSummaryScreen();
}

function showSummaryScreen(){
  var bestPlayer;
  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      var player = {
        id: socket.info.id,
        name: socket.info.nickname,
        character: socket.info.character,
        team: socket.info.team,
        numberOfKills: socket.info.numberOfKills,
        killData: socket.info.killData,
        numberOfTeamKills: socket.info.numberOfTeamKills,
        numberOfDeaths: socket.info.numberOfDeaths,
        numberOfDeathsByOwnTeam: socket.info.numberOfDeathsByOwnTeam
      }

      if(!bestPlayer){
        bestPlayer = player;
      }

      if(bestPlayer.numberOfKills < player.numberOfKills){
        bestPlayer = player;
      }
    }
  });

  server.sendDataToChannel('showSummary', {homename:homeName, homescore:homeScore, awayname:awayName, awayscore:awayScore, bestPlayer:bestPlayer}, 'gamewallApps');
}

function showSummaryScreen2(){

  var topNorway = [];
  var topSweden = [];

  var bestCanon;
  var bestHorse;
  var bestSoldier;

  server.getUserAppsConnected().forEach(function (socket) {
    //console.log('moving player that is a '+socket.info.character);
    var player = {
      id: socket.info.id,
      name: socket.info.nickname,
      character: socket.info.character,
      team: socket.info.team,
      numberOfKills: socket.info.numberOfKills,
      killData: socket.info.killData,
      numberOfTeamKills: socket.info.numberOfTeamKills,
      numberOfDeaths: socket.info.numberOfDeaths,
      numberOfDeathsByOwnTeam: socket.info.numberOfDeathsByOwnTeam
    }

    if(!bestCanon){
      bestCanon = player;
    }
    if(!bestHorse){
      bestHorse = player;
    }
    if(!bestSoldier){
      bestSoldier = player;
    }

    if(bestCanon.killData['canon'] < player.killData['canon']){
      bestCanon = player;
    }
    if(bestHorse.killData['horse'] < player.killData['horse']){
      bestHorse = player;
    }
    if(bestSoldier.killData['soldier'] < player.killData['soldier']){
      bestSoldier = player;
    }



    if(player.team === 'norway'){
      topNorway.push(player);
    }
    else{
      topSweden.push(player);
    }
  });

  topNorway.sort(function(a,b){
    return b.numberOfKills - a.numberOfKills;
  });

  topSweden.sort(function(a,b){
    return b.numberOfKills - a.numberOfKills;
  });


  server.sendDataToChannel('showSummary', {homename:homeName, homescore:homeScore, awayname:awayName, awayscore:awayScore, topNorway:topNorway, topSweden:topSweden, bestSoldier:bestSoldier, bestHorse:bestHorse, bestCanon:bestCanon}, 'gamewallApps');
}



var updateGameLogic = function(){
  // console.log('mode 0');
  gameLogicMode0();
}

function gameLogicMode0(){

}

//infinity lives
function gameLogicMode1(){
  //console.log('mode 1');
  updateAttackTickPlayers();
  movePlayers();
  checkDeadPlayers();
  updateBulletsAndCanonBalls();
  sendUpdatedInformationToGamewalls();
}

//one life
function gameLogicMode2(){
  //console.log('mode 2');
  if(gameMode2Intro){
    interpolateIntroPositions();
    sendUpdatedInformationToGamewalls();
  }else{
    updateAttackTickPlayers();
    movePlayers();
    checkDeadPlayers();
    updateBulletsAndCanonBalls();
    sendUpdatedInformationToGamewalls();
    if(checkIfWeHaveAWinner()){
      resetGameMode2();
    }
  }
}

function resetGameMode2(){
  homeScore = 0;
  awayScore = 0;

  gameMode2CurrentTime = 0;
  gameMode2Intro = true;

  server.getUserAppsConnected().forEach(function (socket) {
    //console.log('moving player that is a '+socket.info.character);
    if(socket.info){
      awakePlayer(socket);
      socket.info.playerLives = gameSettings.lives;
      server.sendDataToChannel('showName', socket.info.id, 'gamewallApps');
    }
  });

  //Set players dead & position
}

function interpolateIntroPositions(){
  if(enoughPlayers){
    gameMode2CurrentTime += (1/24);
    // console.log('Current time: ' + gameMode2CurrentTime);
    if(gameMode2CurrentTime > gameMode2IntroTime){
      // console.log('Done! ' + gameMode2CurrentTime);
      server.getUserAppsConnected().forEach(function (socket) {
        if(socket.info){
          socket.info.alive = true;
          server.sendDataToChannel('showNameStop', socket.info.id, 'gamewallApps');
        }
      });
      gameMode2Intro = false;
    }else{
      server.getUserAppsConnected().forEach(function (socket) {
        if(socket.info){
          var targetX = 300;
          if(socket.info.character === 'soldier'){
            targetX += 300;
          }
          else if(socket.info.character === 'horse'){
            targetX += 150;
          }
          var fac = (gameMode2CurrentTime / gameMode2IntroTime);
          targetX *= fac;

          if(socket.info.team === 'sweden'){
            targetX = 1980 - targetX;
          }
          
          socket.info.x = targetX;
          // socket.movingDirection[0] = 1;
        }
      });
    }
  }else{
    console.log('Not enough players');
  }
}

function enoughPlayers(){
  var arrayOfAliveNorwegians = [];
  var arrayOfAliveSwedes = [];

  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      if(socket.info.alive === true || socket.info.playerLives > 0){
        if(socket.info.team == 'norway'){
          arrayOfAliveNorwegians.push(socket);
        }else{
          arrayOfAliveSwedes.push(socket);
        }
      }
    }
    // console.log('is player team '+socket.info.team);
  });
  if(arrayOfAliveNorwegians.length > 0 && arrayOfAliveSwedes.length > 0){
    // console.log('Some are alive: ' + arrayOfAliveNorwegians.length + ' swedes ' + arrayOfAliveSwedes.length);
    return true;
  }else{
    return false;
  }
}

function checkIfWeHaveAWinner(){
  var playersLeft = enoughPlayers();
  // console.log('Check if we have a winner, players left? ' + playersLeft);
  if(!playersLeft){
    server.sendDataToChannel('gameOver', 'Game over!', 'gamewallApps');
    return true;
  }
  return false;
}

function updateAttackTickPlayers(){
  server.getUserAppsConnected().forEach(function (socket) {
    //console.log('moving player that is a '+socket.info.character);
    if(socket.info){
      if(socket.info.attack){
        socket.info.attackTick++;
      }
      if(socket.info.attackTick > 20){
        socket.info.attack = false;
        socket.info.attackTick = 0;
      }
    }
  });
}

function movePlayers(){
  //console.log('movePlayers');
  server.getUserAppsConnected().forEach(function (socket) {
    //console.log('moving player that is a '+socket.info.character);
    if(socket.info){
      if(socket.info.character){
        playerInWaterUpdate(socket);
        socket.info.movePlayer();
      }
    }
  });
}

function moveSoldier(socket){
  //console.log('moveSoldier');
  socket.info.x += socket.info.playerWalkValueX * socket.info.movingDirection[0] * speedFactor;
  if(socket.info.x > (1920 - 100) || socket.info.x < (0 + 100)){
    socket.info.x -= socket.info.playerWalkValueX * socket.info.movingDirection[0] * speedFactor;
  }
  socket.info.y += socket.info.playerWalkValueY * socket.info.movingDirection[1] * speedFactor;
  if(socket.info.y > (1080 - 50) || socket.info.y < (0 + 200)){
    socket.info.y -= socket.info.playerWalkValueY * socket.info.movingDirection[1] * speedFactor;
  }
}

function moveHorse(socket){
  //console.log('moveHorse');
  socket.info.x += socket.info.playerWalkValueX * socket.info.movingDirection[0] * speedFactor;
  if(socket.info.x > 1920 + 100){
    socket.info.x = 0 - 100;
  }
  else if(socket.info.x < 0 - 100){
    socket.info.x = 1920 + 100;
  }
  socket.info.y += socket.info.playerWalkValueY * socket.info.movingDirection[1] * speedFactor;
  if(socket.info.y > (1080 - 50) || socket.info.y < (0 + 200)){
    socket.info.y -= socket.info.playerWalkValueY * socket.info.movingDirection[1] * speedFactor;
  }
}

function moveCanon(socket){
  //console.log('moveCanon');
  socket.info.y += socket.info.playerWalkValueY * socket.info.movingDirection[1] * speedFactor;
  if(socket.info.y > (1080 - 50) || socket.info.y < (0 + 200)){
    socket.info.y -= socket.info.playerWalkValueY * socket.info.movingDirection[1] * speedFactor;
  }
}

function playerInWaterUpdate(socket){
  if(inWaterXPos(socket.info.x) && inWaterYPos(socket.info.y)){
    socket.info.playerWalkValueX = socket.info.originalPlayerWalkValueX / 3;
    socket.info.playerWalkValueY = socket.info.originalPlayerWalkValueY / 3;
    socket.info.inWater = true;
  }
  else{
    socket.info.playerWalkValueX = socket.info.originalPlayerWalkValueX;
    socket.info.playerWalkValueY = socket.info.originalPlayerWalkValueY;
    socket.info.inWater = false;
  }
}

function inWaterXPos(x){
  if(x > waterStartX && x < waterEndX){
    return true;
  }
  return false;
}

function inWaterYPos(y){
  if(y < bridgeStartY || (y > bridgeEndY && y < bridge2StartY) || y > bridge2EndY){
    return true;
  }
  return false;
}

function checkDeadPlayers(){
  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      if(socket.info.alive === false){
        var tickPlayer = true;
        if(gameMode.chosenMode == 2 && socket.info.playerLives <= 0){
          tickPlayer = false;
        }
        if(tickPlayer){
          socket.info.aliveTick += 1;
          if(socket.info.aliveTick > 72){
            awakePlayer(socket);
            if(gameMode.chosenMode == 2){
              socket.info.playerLives -= 1;
              // console.log('Player has ' + socket.info.playerLives + ' lives left');
            }
          }
        }
      }
    }
  });
}

function awakePlayer(socket){
  socket.info.alive = true;
  socket.info.aliveTick = 0;
  /*
  var characterInfo = getCharacter(socket);
  var width = getWidth(characterInfo.character);
  var height = getHeight(characterInfo.character);
  socket.info.character = characterInfo.character;
  socket.info.playerWalkValueX = characterInfo.walkValueX;
  socket.info.playerWalkValueY = characterInfo.walkValueY;
  socket.info.originalPlayerWalkValueX = characterInfo.walkValueX;
  socket.info.originalPlayerWalkValueY = characterInfo.walkValueY;
  socket.info.movingDirection = [0, 0];
  socket.info.width = width;
  socket.info.height = height;
  var newStartPos = getStartPosition(socket.info.team, socket.info.character);
  socket.info.x = newStartPos[0];
  socket.info.y = newStartPos[1];
  */

  if(gameSettings.charSelection == 'Free'){
    var team = getTeam(socket);
    console.log('Dead player char sel with team ' + team);

    var activeSoldiers = 0, activeCavalery = 0, activeCanons = 0;
    var maxSoldiers = 'x', maxCavalery = 'x', maxCanons = 'x';
    if(gameSettings.ratio == 'Limited'){
      activeSoldiers = getUnitTypeForTeam(team, 'soldier');
      maxSoldiers = getMaxUnitType(team, 'soldier');
      activeCavalery = getUnitTypeForTeam(team, 'horse');
      maxCavalery = getMaxUnitType(team, 'horse');
      activeCanons = getUnitTypeForTeam(team, 'canon');
      maxCanons = getMaxUnitType(team, 'canon');
    }
    var msg = {team:team, activeSoldiers:activeSoldiers, maxSoldiers:maxSoldiers, activeCavalery:activeCavalery, maxCavalery:maxCavalery, activeCanons:activeCanons, maxCanons:maxCanons};
    server.sendDataToSocket(socket, 'characterSelection', msg);
  }else{
    console.log('charselection is not free');
    createNewPlayer(socket);

    server.sendDataToChannel('newPlayer', {id:socket.info.id, name:socket.info.nickname, team:socket.info.team, character:socket.info.character, x:socket.info.x, y:socket.info.y, movingDirection:socket.info.movingDirection}, 'gamewallApps');
    server.sendDataToSocket(socket, 'setup', {team:socket.info.team, character:socket.info.character});
  }
  
  
}



function updateBulletsAndCanonBalls(){
  updateCanonBallsTemp();

  for(var i = 0; i < bullets.length; i++){
    var localBullet = bullets[i];
    localBullet.tick += 1;
    if(localBullet.tick > localBullet.timeToLive){
      bullets.splice(i,1);
    }
    else{
      localBullet.x += localBullet.speed * speedFactor;
      checkForCollisionBetweenBulletsAndPlayers(localBullet);
    }
  }
  for(var i = 0; i < canonBalls.length; i++){
    var localCanonBall = canonBalls[i];
    localCanonBall.tick += 1;
    if(localCanonBall.tick > localCanonBall.timeToLive){
      canonBalls.splice(i,1);
      checkForCollisionBetweenCanonBallAndPlayers(localCanonBall);
      var inWater = false;
      if(inWaterXPos(localCanonBall.x) && inWaterYPos(localCanonBall.y)){
        inWater = true;
      }
      server.sendDataToChannel('explodeCanonBall', {x:localCanonBall.x, y:localCanonBall.y, inWater:inWater}, 'gamewallApps');
      canonExplosions.push({x:localCanonBall.x, y:localCanonBall.y, inWater:inWater});
    }
    else{
      localCanonBall.x += localCanonBall.speed * speedFactor;
    }
  }
}

function checkForCollisionBetweenCanonBallAndPlayers(projectile){
  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      if(projectile.x - (projectile.width / 2) < socket.info.x + (socket.info.width / 2) && projectile.x + (projectile.width / 2) > socket.info.x - (socket.info.width / 2)){
        if(projectile.y - (projectile.height / 2) < socket.info.y + (socket.info.height / 2) && projectile.y + (projectile.height / 2) > socket.info.y - (socket.info.height / 2)){
          if(projectile.id !== socket.info.id && socket.info.alive){
            // console.log('we have a hit!');
            killPlayer(socket);
            removeProjectile(projectile.id);
            updateNumberOfKills(socket, projectile.id);
            updateOverallScore();
          }
        }
      }
    }
  });
}

function checkForCollisionBetweenBulletsAndPlayers(projectile){
  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      if(projectile.x - (projectile.width / 2) < socket.info.x + (socket.info.width / 2) && projectile.x + (projectile.width / 2) > socket.info.x - (socket.info.width / 2)){
        if(projectile.y - (projectile.height / 2) < socket.info.y + (socket.info.height / 2) && projectile.y + (projectile.height / 2) > socket.info.y - (socket.info.height / 2)){
          if(projectile.id !== socket.info.id && projectile.team !== socket.info.team && socket.info.alive){
            // console.log('we have a hit!');
            killPlayer(socket);
            removeProjectile(projectile.id);
            updateNumberOfKills(socket, projectile.id);
            updateOverallScore();
          }
        }
      }
    }
  });
}

function updateNumberOfKills(victim, killerID){
  var killer;
  server.getUserAppsConnected().forEach(function (socket) {
    if(socket.info){
      if(socket.info.id === killerID){
        killer = socket;
      }
    }
  });

  if(killer.info.team === victim.info.team){
    victim.info.numberOfDeathsByOwnTeam += 1;
    killer.info.numberOfTeamKills += 1;
    killer.info.killData[killer.info.character] -= 1;
    if(killer.info.team === 'norway'){ // 1 = norge, -1 = sverige
      awayScore++;
    }
    else{
      homeScore++;
    }
  }
  else{
    killer.info.numberOfKills += 1;
    killer.info.killData[killer.info.character] += 1;
    checkNumberOfKills(killer);
    if(killer.info.team === 'norway'){ // 1 = norge, -1 = sverige
      homeScore++;
    }
    else{
      awayScore++;
    }
  }
}

function checkNumberOfKills(player){
  if(player.info.numberOfKills === 3){
    server.sendDataToChannel('killspree', {playerid:player.id, text:'Nice!'}, 'gamewallApps');
  }
  else if(player.info.numberOfKills === 5){
    server.sendDataToChannel('killspree', {playerid:player.id, text:'Monsterkill!'}, 'gamewallApps');
  }
  else if(player.info.numberOfKills === 7){
    server.sendDataToChannel('killspree', {playerid:player.id, text:'Insane!'}, 'gamewallApps');
  }
  else if(player.info.numberOfKills === 10){
    server.sendDataToChannel('killspree', {playerid:player.id, text:'Godlike!'}, 'gamewallApps');
  }
}

function removeProjectile(projectileId){
  for(var i = 0; i < bullets.length; i++){
    var localBullet = bullets[i];
    if(localBullet.id === projectileId){
      bullets.splice(i,1);
    }
  }
  for(var i = 0; i < canonBalls.length; i++){
    var localCanonBall = canonBalls[i];
    if(localCanonBall.id === projectileId){
      canonBalls.splice(i,1);
    }
  }
}

function killPlayer(socket){
  //console.log('killPlayer');
  for(var i = 0; i < canonBallsTemp.length; i++){
    var localCanonBall = canonBallsTemp[i];
    if(localCanonBall.id === socket.info.id){
      canonBallsTemp.splice(i,1);
    }
  }
  socket.info.alive = false;
  socket.info.playerIsActive = false;
  socket.info.aliveTick = 0;
  socket.info.numberOfDeaths += 1;
  tombstones.push({x:socket.info.x, y:socket.info.y});
  server.sendDataToChannel('killPlayer', socket.info.id, 'gamewallApps');
  server.sendEventToSocket(socket, 'killPlayer');
}


function sendUpdatedInformationToGamewalls(){
  //console.log('sendingUpdatedInformation');
  var clientsArray = [];
  server.getUserAppsConnected().forEach(function (socket) {
    //console.log('client');
    //console.log(socket.info.id);
    //console.log(socket.info);
    if(socket.info){
      var client = [socket.info.id, socket.info.x, socket.info.y, socket.info.movingDirection, socket.info.inWater];
      clientsArray.push(client);
    }
  });
  server.sendDataToChannel('updatePlayers', clientsArray, 'gamewallApps');

  server.sendDataToChannel('updateProjectiles', [bullets, canonBalls], 'gamewallApps');
}


function updateOverallScore(){
  server.sendDataToChannel('updateScore', {homeName:homeName, homeScore:homeScore, awayName:awayName, awayScore:awayScore}, 'gamewallApps');
}


function startWithGameSettings(){
  var mode = 'Team Deathmatch';
  var level = 'Grasslands';
  var time = 5 * 60; //send number of seconds
  var speed = 120;
  var charSelection = 'Random';
  var ratio = 'Free';
  var lives = 10;
  var settingsMsg = { mode:mode, level: level, time:time, speed:speed, charSelection:charSelection, ratio:ratio, lives:lives };

  gameMode.isSelected = true;
  gameSettings = settingsMsg;
  speedFactor = gameSettings.speed / 100;

  resetScore();
  resetPlayers();
  // console.log(settings);
  if(gameSettings.mode == 'Team Deathmatch'){
    gameMode.chosenMode = 1;
    updateGameLogic = function(){
      // console.log('Setting mode 1');
      gameLogicMode1();
    }
  }
  else if(gameSettings.mode == 'Rounds'){
    gameMode.chosenMode = 2;
    resetGameMode2();
    // console.log('Setting mode 2');
    updateGameLogic = function(){
      //gameLogicMode2();
      gameLogicMode2();
    }
  }
  //socket.emit('chooseGameMode', settingsMsg);
}

/////////////////////////////////////////////////
//////             Init server            ///////
/////////////////////////////////////////////////
startWithGameSettings();
server.init();

