var words = [];
var fill = d3.scale.category20();
var layout;

var rand = 0;
var maxWordSize = 200;
var minWordSize = 20;

$(document).ready(function(){
  
  connectToLocalServer();

  addBinding('word', function (data){
    console.log('Getting word');
    handleMessage(data);
  });

  addBinding('deleteWord', function (data){
    console.log('deleting word');
    deleteWord(data);  
  });

  addBinding('history', function (data){
    for(var i = 0; i < data.length; i++){
      handleMessage(data[i]); 
    }
  });

  var qrurl = 'http://' + location.host + '/index.html';
  var qrImg = '<img id="qrcodeimg" src="http://qrickit.com/api/qr?d='+qrurl+'&addtext='+qrurl+'&txtcolor=442EFF&fgdcolor=000000&bgdcolor=FFFFFF&qrsize=150&t=p&e=m" onmouseover="showQRCode()" onmouseout="closeQRCode()">';
  $('#qrcode').prepend(qrImg);

  setupGameWall();
});


function showQRCode(){
  console.log('qr code');
  var windowHeight = $(document).height();
  var windowWidth = $(document).width();
  var qrsize = Math.min(windowWidth,windowHeight,900);
  console.log('height: '+windowHeight+', width: '+windowWidth);
  $('#qrcodeimg').animate({
    opacity: 1,
    height: qrsize+'px',
    width: qrsize+'px'
  }, 1000, function() {
    // Animation complete.
  });
}

function closeQRCode(){
  $('#qrcodeimg').animate({
    opacity: 1,
    height: '150px',
    width: '150px'
  }, 1000, function() {
    // Animation complete.
  });
}

function deleteWord(word){
  console.log('Getting word: ' + word.text + ' mentioned ' + word.numberMentioned);
  var text = word.text;
  
  if(words.length === 0){
    return;
  }
  else{
    for(var i = 0; i < words.length; i++){
      if(words[i].text === text){
        words.splice(i,1);
        break;
      }
    }
  }
  
  calculateWordSizes(words);
  layout.words(words);

  document.getElementsByTagName('BODY')[0].innerHTML = '';
  setupGameWall();
}



function handleMessage(word){
  console.log('Getting word: ' + word.text + ' mentioned ' + word.numberMentioned);
  var text = word.text;
  var sizeInc = 5;
  


  if(words.length === 0){
    var newWord = {text:text, size:maxWordSize, numberMentioned:word.numberMentioned};
    words.push(newWord);
  }
  else{
    var foundWord = false;
    for(var i = 0; i < words.length; i++){
      if(words[i].text === text){

        words[i].numberMentioned += 1;
        foundWord = true;
        break;
      }
    }
    if(!foundWord){
      var newWord = {text:text, size:maxWordSize, numberMentioned:word.numberMentioned};
      words.push(newWord);
    }
  }
  
  calculateWordSizes(words);
  layout.words(words);

  document.getElementsByTagName('BODY')[0].innerHTML = '';
  setupGameWall();
}

function calculateWordSizes(words){
  var maxTimesMentioned = 0;
  var minTimesMentioned = 9999;
  var size = maxWordSize;
  for(var i = 0; i < words.length; i++){
    var word = words[i];
    if(word.numberMentioned > maxTimesMentioned){
      maxTimesMentioned = word.numberMentioned;
    }
    if(word.numberMentioned < minTimesMentioned){
      minTimesMentioned = word.numberMentioned;
    }
  }

  for(var i = 0; i < words.length; i++){
    var word = words[i];
    var mentioned = word.numberMentioned;
    // var fs = minFontsize + (maxFontsize * (countI - minCount)) / (maxCount - minCount);
    size = maxWordSize - ((maxWordSize - minWordSize) * (maxTimesMentioned - mentioned)) / (maxTimesMentioned);

    word.size = size;
  }
 
  //console.log('Word mentioned ' + mentioned + '/' + maxTimesMentioned + ' = ' + size);
  

  return size;
}

function setupGameWall(){
  console.log('Setup gamewall');
  layout = d3.layout.cloud().size([1920, 1080]);

  layout.words(words);
  layout.rotate(function() { return ~~(rand) * 90; });
  layout.font("Impact");
  layout.fontSize(function(d) { return d.size; });
  layout.on("end", draw);
  layout.start();
}


function draw(words) {
  d3.select("body").append("svg")
      .attr("width", 1920)
      .attr("height", 1080)
    .append("g")
      .attr("transform", "translate(" + (1920/2) + "," + (1080/2) + ")")
    .selectAll("text")
      .data(words)
    .enter().append("text")
      .style("font-size", function(d) { return d.size + "px"; })
      .style("font-family", "Impact")
      .style("fill", function(d, i) { return fill(i); })
      .attr("text-anchor", "middle")
      .attr("transform", function(d) {
        return "translate(" + [d.x, d.y] + ")rotate(" + d.rotate + ")";
      })
      .text(function(d) { return d.text; });
}
